function [coor,maxVal,im1,oclu]=impatron_2D(im,template,coor,oclu)


% [coor,maxVal,im1,oclu]=impatron_2D(im,template,coor,oclu);
% Variables de salida:
% coor: vector con las coordenadas [x,y] del punto donde se alcanza el mayor
%      valor del filtro
% maxVal: valor mximo alcanzado por el filtro
% im1: video con el patrn marcado
% oclu: indica si hay oclusin en el fotograma analizado
% Variables de entrada:
% im: nombre de la imagen sobre la que buscar un patrn
% template: imagen que constituye el patrn de bsqueda
% coor: coordenadas donde estaba el patrn en el fotograma anterior
% oclu: indica si ha habido oclusin en el fotograma anterior


% inicializacin de variables
maxVal=-1;
ndb=40;      % variacin permitida en el valor medio de los pixeles
pp=10;        % nmero de pixeles que definen una vecindad

% lectura de las imgenes
a=im;


% clculo de las dimensiones de las imgenes

[ng,mg,lg]=size(a);
[np,mp,lp]=size(template);


if (lg~=lp)
    display('La imagen y el patrn deben ser ambas imgenes en escala de grises o imgenes en color\n');
    return;
end



% transformamos las imgenes a double

a=double(a); template=double(template);


% inicializacin de las matrices que guardan los productos escalares

q=zeros(ng-np+1,mg-mp+1);
p=zeros(ng-np+1,mg-mp+1,lg);
valor_medio=zeros(ng-np+1,mg-mp+1,lg);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   CASO EN QUE HAY OCLUSIN EN EL FOTOGRAMA ANTERIOR
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if oclu==1

    display('Oclusin en el Fotograma Anterior o es el Primer Fotograma')


    for k=1:lg

        % calculamos el valor medio de los pixeles del template

        valor_medio_template(k)=sum(sum(template(:,:,k)))/np/mp;


        % normalizamos el template

        template_n(1:np,1:mp,k)=template(1:np,1:mp,k)/(norm(template(1:np,1:mp,k),'fro')+10^(-10));

        % recorremos la matriz a y para cada submatriz de tamao npxmp con filas y columnas correlativas
        % hacemos el producto escalar en la norma frobenius con el template tras haber previamente normalizado ambas
        % matrices

        for i=1:ng-np+1
            for j=1:mg-mp+1

                % vamos tomando muestras en la matriz a del mismo tamao que el
                % template

                muestra(1:np,1:mp)=a(i:i+np-1,j:j+mp-1,k);

                % calculamos el valor medio de la muestra

                valor_medio(i,j,k)=sum(sum((muestra)))/np/mp;


                % normalizamos la muestra

                muestra=muestra/(norm(muestra,'fro')+10^(-10));


                % calculamos el producto escalar en norma frobenius de el template normalizado
                % con la muestra normalizada

                p(i,j,k)=sum(sum(muestra.*template_n(:,:,k)));


                q(i,j)=q(i,j)+p(i,j,k);

            end
        end

    end


    % Nos quedamos con las muestras cuyo valor medio para cada una de las bandas
    % diste menos de ndb db del valor medio del template para esa banda


    [pos1x,pos1y]=find(abs(valor_medio(:,:,1)-valor_medio_template(1))<ndb);
    [pos2x,pos2y]=find(abs(valor_medio(:,:,2)-valor_medio_template(2))<ndb);
    [pos3x,pos3y]=find(abs(valor_medio(:,:,3)-valor_medio_template(3))<ndb);


    % Nos quedamos tambin con las posiciones donde se da un valor del
    % filtro mayor que un sesgo

    sesgo=2.998;
    [pos4x,pos4y]=find(q>sesgo);


    % Matrices que contienen los ndices factibles
    pos1=[pos1x pos1y];
    pos2=[pos2x  pos2y];
    pos3=[pos3x pos3y];
    pos4=[pos4x pos4y];

   
    
    % Nos quedamos con las posiciones que aparecen en todas las listas

    pos=[];
    for i=1:size(pos4,1)
        for j=1:size(pos3,1)
            if pos4(i,:)-pos3(j,:)==[0,0]
                for k=1:size(pos2,1)
                    if pos4(i,:)-pos2(k,:)==[0,0]
                        for l=1:size(pos1,1)
                            if  pos4(i,:)-pos1(l,:)==[0,0]
                                pos=[pos;[pos4(i,1),pos4(i,2)]];
                            end
                        end
                    end
                end
            end
        end
    end
    
  

    % Si la lista de posiciones posibles no es vacia, suponemos que hemos
    % encontrado el patrn, guardamos las coordenadas de la posicin donde
    % lo hemos encontrado y dibujamos un rectngulo en la imagen sealando
    % el patrn


    if ~isempty(pos)

        % Suponemos que hemos encontrado el patrn y por tanto
        % ponemos oclu a 0

        oclu=0;

        % Buscamos de las posibles posiciones la que da mayor valor de
        % filtro
        
        coor(1)=pos(1,1); coor(2)=pos(1,2);
        maxVal=q(pos(1,1),pos(1,2));
        for i=2:size(pos,1)
            aux=q(pos(i,1),pos(i,2));
            if aux>maxVal
                coor(1)=pos(i,1); coor(2)=pos(i,2);
                maxVal=q(pos(i,1),pos(i,2));
            end
        end

        % Pintamos el rectngulo rojo alrededor del patrn

        a(coor(1):coor(1)+np,coor(2)-2:coor(2)+2,1)=255;
        a(coor(1):coor(1)+np,coor(2)-2:coor(2)+2,2)=0;
        a(coor(1):coor(1)+np,coor(2)-2:coor(2)+2,3)=0;

        a(coor(1):coor(1)+np,coor(2)+mp-2:coor(2)+mp+2,1)=255;
        a(coor(1):coor(1)+np,coor(2)+mp-2:coor(2)+mp+2,2)=0;
        a(coor(1):coor(1)+np,coor(2)+mp-2:coor(2)+mp+2,3)=0;

        a(coor(1)-2:coor(1)+2,coor(2):coor(2)+mp,1)=255;
        a(coor(1)-2:coor(1)+2,coor(2):coor(2)+mp,2)=0;
        a(coor(1)-2:coor(1)+2,coor(2):coor(2)+mp,3)=0;

        a(coor(1)+np-2:coor(1)+np+2,coor(2):coor(2)+mp,1)=255;
        a(coor(1)+np-2:coor(1)+np+2,coor(2):coor(2)+mp,2)=0;
        a(coor(1)+np-2:coor(1)+np+2,coor(2):coor(2)+mp,3)=0;

    end

    
else   % oclu=0;   Caso en que conocemos las coordenadas del patrn anteriormente


    display('Sin Oclusin en el Fotograma Anterior')


    for k=1:lg

        % calculamos el valor medio de los pixeles del template

        valor_medio_template(k)=sum(sum(template(:,:,k)))/np/mp;


        % normalizamos el template

        template_n(1:np,1:mp,k)=template(1:np,1:mp,k)/(norm(template(1:np,1:mp,k),'fro')+10^(-10));

        % recorremos un entorno de la posicin anterior del patrn en la matriz a
        % y para cada submatriz de tamao npxmp con filas y columnas correlativas
        % hacemos el producto escalar en la norma frobenius con el template tras
        % haber previamente normalizado ambas matrices

        for i=coor(1)-pp:coor(1)+pp
            for j=coor(2)-pp:coor(2)+pp

                if i+np-1<=ng & j+mp-1<=mg    % Podemos tomar la muestra dentro de la imagen

                    % vamos tomando muestras en la matriz a del mismo tamao que el
                    % template

                    muestra(1:np,1:mp)=a(i:i+np-1,j:j+mp-1,k);
      

                    % calculamos el valor medio de la muestra

                    valor_medio(i,j,k)=sum(sum((muestra)))/np/mp;


                    % normalizamos la muestra

                    muestra=muestra/(norm(muestra,'fro')+10^(-10));


                    % calculamos el producto escalar en norma frobenius de el template normalizado
                    % con la muestra normalizada

                    p(i,j,k)=sum(sum(muestra.*template_n(:,:,k)));

                   

                    q(i,j)=q(i,j)+p(i,j,k);

                else
                    q(i,j)=0;

                end


            end
        end

    end


    % Nos quedamos con las muestras cuyo valor medio para cada una de las bandas
    % diste menos de ndb db del valor medio del template para esa banda


    [pos1x,pos1y]=find(abs(valor_medio(coor(1)-pp:coor(1)+pp,coor(2)-pp:coor(2)+pp,1)-valor_medio_template(1))<ndb);
    [pos2x,pos2y]=find(abs(valor_medio(coor(1)-pp:coor(1)+pp,coor(2)-pp:coor(2)+pp,2)-valor_medio_template(2))<ndb);
    [pos3x,pos3y]=find(abs(valor_medio(coor(1)-pp:coor(1)+pp,coor(2)-pp:coor(2)+pp,3)-valor_medio_template(3))<ndb);


    pos1x=pos1x+coor(1)-pp-1;
    pos1y=pos1y+coor(2)-pp-1;
    pos2x=pos2x+coor(1)-pp-1;
    pos2y=pos2y+coor(2)-pp-1;
    pos3x=pos3x+coor(1)-pp-1;
    pos3y=pos3y+coor(2)-pp-1;
    
%     % Nos quedamos tambin con las posiciones donde se da un valor del
%     % filtro mayor que un sesgo
% 
%     sesgo=2.8;
%     [pos4x,pos4y]=find(q(coor(1)-pp:coor(1)+pp,coor(2)-pp:coor(2)+pp)>sesgo);
% 
%     pos4x=pos4x+coor(1)-pp+1;
%     pos4y=pos4y+coor(2)-pp+1;
% 
%     % Matrices que contienen los ndices factibles
%     pos1=[pos1x pos1y];
%     pos2=[pos2x  pos2y];
%     pos3=[pos3x pos3y];
%     pos4=[pos4x pos4y];

   
    
%     % Nos quedamos con las posiciones que aparecen en todas las listas
% 
%     pos=[];
%     for i=1:size(pos4,1)
%         for j=1:size(pos3,1)
%             if pos4(i,:)-pos3(j,:)==[0,0]
%                 for k=1:size(pos2,1)
%                     if pos4(i,:)-pos2(k,:)==[0,0]
%                         for l=1:size(pos1,1)
%                             if  pos4(i,:)-pos1(l,:)==[0,0]
%                                 pos=[pos;[pos4(i,1),pos4(i,2)]];
%                             end
%                         end
%                     end
%                 end
%             end
%         end
%     end
%     
    
    
    
    % Matrices que contienen los ndices factibles
    pos1=[pos1x pos1y];
    pos2=[pos2x  pos2y];
    pos3=[pos3x pos3y];
    
    
    
   

    % Nos quedamos con las posiciones que aparecen en todas las listas

   

    pos=[];
    for i=1:size(pos1,1)
        for j=1:size(pos2,1)
            if pos1(i,:)-pos2(j,:)==[0,0]
                  for k=1:size(pos3,1)
                        if pos1(i,:)-pos3(k,:)==[0,0]
                             pos=[pos;[pos1(i,1),pos1(i,2)]];
                        end
                  end
            end
        end
    end

 

    if ~isempty(pos)
% 
% % Buscamos en qu posicin se da el menor cambio de coordenadas
% minimo_coor=abs(coor(1)-pos(1,1))+abs(coor(2)-pos(1,2))
% ind_min_coor=[pos(1,1),pos(1,2)];
% display('eh1');
% maxVal=q(ind_min_coor(1),ind_min_coor(2))
% display('eh2');
% for i=2:size(pos,1)
%     aux=abs(coor(1)-pos(i,1))+abs(coor(2)-pos(i,2))
%     if aux<=minimo_coor
%         ind_min_coor=[pos(i,1),pos(i,2)];
%         minimo_coor=aux;
%         maxVal=q(ind_min_coor(1),ind_min_coor(2));
%     end
% end
%         coor=ind_min_coor;
%         
        
        
        % Entre las posiciones posibles nos quedamos elegimos en la que el
        % valor del filtro es mayor

        coor(1)=pos(1,1); coor(2)=pos(1,2);
        maxVal=q(pos(1,1),pos(1,2));
        for i=2:size(pos,1)
            aux=q(pos(i,1),pos(i,2));
            if aux>maxVal
                coor(1)=pos(i,1); coor(2)=pos(i,2);
                maxVal=q(pos(i,1),pos(i,2));
            end
        end


        % Pintamos el rectngulo rojo alrededor del patrn

        a(coor(1):coor(1)+np,coor(2)-2:coor(2)+2,1)=255;
        a(coor(1):coor(1)+np,coor(2)-2:coor(2)+2,2)=0;
        a(coor(1):coor(1)+np,coor(2)-2:coor(2)+2,3)=0;

        a(coor(1):coor(1)+np,coor(2)+mp-2:coor(2)+mp+2,1)=255;
        a(coor(1):coor(1)+np,coor(2)+mp-2:coor(2)+mp+2,2)=0;
        a(coor(1):coor(1)+np,coor(2)+mp-2:coor(2)+mp+2,3)=0;

        a(coor(1)-2:coor(1)+2,coor(2):coor(2)+mp,1)=255;
        a(coor(1)-2:coor(1)+2,coor(2):coor(2)+mp,2)=0;
        a(coor(1)-2:coor(1)+2,coor(2):coor(2)+mp,3)=0;

        a(coor(1)+np-2:coor(1)+np+2,coor(2):coor(2)+mp,1)=255;
        a(coor(1)+np-2:coor(1)+np+2,coor(2):coor(2)+mp,2)=0;
        a(coor(1)+np-2:coor(1)+np+2,coor(2):coor(2)+mp,3)=0;

    else



        maxVal=-1;
        oclu=1;

    end

end

im1=uint8(a);



